/*
 * Copyright (C) 2012 Brockmann Consult GmbH (info@brockmann-consult.de)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option)
 * any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, see http://www.gnu.org/licenses/
 */

package org.esa.snap.myop;

import org.esa.snap.core.datamodel.Product;
import org.esa.snap.core.gpf.Operator;
import org.esa.snap.core.gpf.OperatorException;
import org.esa.snap.core.gpf.OperatorSpi;
import org.esa.snap.core.gpf.annotations.OperatorMetadata;
import org.esa.snap.core.gpf.annotations.Parameter;
import org.esa.snap.core.gpf.annotations.SourceProduct;
import org.esa.snap.core.gpf.annotations.TargetProduct;

import com.bc.ceres.core.ProgressMonitor;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Logger;

//import org.esa.snap.runtime.EngineConfig;

/**
 * The <code>MyMerisPointOp</code> serves as a template for other MERIS L1B/OLCI point ops.
 *
 * @author Norman
 */


@OperatorMetadata(
        alias = "TestOperator",
        category = "Optical Processing/Thematic <Water|Land> Processing",
        description = "The '<your operator name here>' operator retrieves '<your operator product(s) here>' from MERIS/OLCI L1B products.",
        authors = "<your name here>",
        copyright = "<your copyright info here>")
public class MyTestOperator extends Operator {

//    private boolean hasValidPixelExpression;

    private final String operatorName = "TestOperator";

    @SourceProduct(alias = "source", description = "The source product.")
    private Product sourceProduct;

    @TargetProduct(description = "The target product.")
    private Product targetProduct;

    @Parameter(defaultValue = "1.0", description = "Hodnota")
    private double val;

    @Parameter(defaultValue = "+", description = "Matematicky operator (+,-,*,/)")
    private String mathOperator;

    @Parameter(defaultValue = "c:\\Python27\\python.exe", description = "Cesta k spustaciemu programu. Napr. (c:\\Python27\\python.exe)")
    private String execPath;

    @Parameter(defaultValue = "d:\\work\\SnapTools\\test\\raster_test.py", description = "Cesta k programu. Napr. (d:\\work\\SnapTools\\test\\raster_test.py)")
    private String execFile;

    protected String inFilePath;
    protected String outFilePath;





    @Override
    public void initialize(){
        Product sProduct = this.getSourceProduct();
        File inFile = sProduct.getFileLocation();
        this.inFilePath = inFile.getAbsolutePath();


//    todo    check input & output to be tiff
//        set targetProduct
    }
    @Override
    public void doExecute(ProgressMonitor pm) {

        this.logMessage("doExecute hit.");

        String cmd = this.getCommandString();

        String taskMessage = String.format("Executing tool command %s ...", cmd);


        logMessage(taskMessage);

        pm.beginTask(taskMessage,1);

        this.executeToolCommand(cmd, pm);

        this.logMessage("Command executed successfully.");

        pm.worked(1);
        pm.done();

    }

    private void logMessage(String message){
//        Logger logger = EngineConfig.instance().logger();
//        String timeStamp = new SimpleDateFormat("yyyy-MM-dd-HH:mm:ss.SSS").format(new Date());
//
//        String fullMess = this.operatorName+" "+timeStamp +": "+message;
//        logger.fine(fullMess);
    }

    private void executeToolCommand (String cmd, ProgressMonitor pm) {

        try{
            final Process process = Runtime.getRuntime().exec(cmd);

            new Thread(new Runnable() {
                public void run() {
                    BufferedReader input =   new BufferedReader(new InputStreamReader(process.getInputStream()));

                    String line = null;
                    try{
                    while ((line = input.readLine())!= null){

                    }
                    } catch (IOException e){
                        throw new OperatorException(e);
                    }

                }


            }).start();

            process.waitFor();
        }
        catch (IOException e){
            throw new OperatorException(e);
        } catch (InterruptedException e){
            throw new OperatorException(e);
        }

    }

    protected String getCommandString(){

        Product sourceProduct = this.getSourceProduct();
        File inFile = sourceProduct.getFileLocation();
        String inFilePath = inFile.getAbsolutePath();

        String outFilePath = this.getOutFilePath();

        StringBuilder sb = new StringBuilder();
        sb.append(this.execPath);
        sb.append(" ");
        sb.append(this.execFile);
        sb.append(" -i ");
        sb.append(inFilePath);
        sb.append(" -v ");
        sb.append(this.val);
        sb.append(" -e ");
        sb.append(this.mathOperator);
        sb.append(" -o ");
        sb.append(outFilePath);

        return sb.toString();

    }

    private String getOutFilePath(){

        String outFilePath = null;
        if(this.targetProduct != null){
            Product targetProduct = this.getTargetProduct();
            File outFile = targetProduct.getFileLocation();
            outFilePath = outFile.getAbsolutePath();
        }


        Product sourceProduct = this.getSourceProduct();
        File sourceFile = sourceProduct.getFileLocation();
        String sourceFileName = sourceFile.getName();

        if(outFilePath != null && !outFilePath.equals("")){

            if (outFilePath.toLowerCase().endsWith(".tif")){
                return outFilePath;
            }else{

                return getTempPath(sourceFileName);
            }
        }else{
            return getTempPath(sourceFileName);
        }


    }

    private String getTempPath (String fileName){
        String suffix = this.operatorName;

        String timeStamp = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        String tempPath = System.getProperty("java.io.tmpdir");

        String fileExt = getFileExtension(fileName);
        String newFileName;
        if (fileExt.length() > 0){
         newFileName = fileName.substring(0, (fileName.length() - fileExt.length()-1))+"_"+suffix+"_"+timeStamp+"."+fileExt;


        }else{
         newFileName = fileName+"_"+suffix+"_"+timeStamp;
        }

        String tempFilePath = String.format("%s\\%s",tempPath,newFileName);

        return tempFilePath;



    }

    public String getFileExtension(String filename) {

        String[] nameSplit = filename.split("\\.");

        if (nameSplit.length > 0){
            return nameSplit[nameSplit.length -1 ];
        }else{
            return "";
        }

    }



    public static class Spi extends OperatorSpi {

        public Spi() {
            super(MyTestOperator.class);
        }

    }
}
